"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Endpoints = void 0;
const Constants_1 = require("./Constants");
class Endpoints {
    static get base() {
        return Constants_1.APIBase;
    }
    /**
     * @example
     * "/timelines/users/:userId"
     * "/timelines/users/by_nick/:userNick"
     */
    static userTimeline(query, byNick = false) {
        return this.timelines + this.user(query, byNick);
    }
    /**
     * @param query The user id or nick
     * @param byNick Whether to search by nick (Default: false)
     * @example
     * "/users/:userId"
     * "/users/by_nick/:userNick"
     */
    static user(query, byNick = false) {
        return "/users/" + (byNick ? `by_nick/${query}` : query);
    }
    /**
     * @param user The user id or nick
     * @param byNick Whether the user is by nick (Default: false)
     * @example
     * "/users/:userId/guests"
     * "/users/by_nick/:userNick/guests"
     */
    static guests(user, byNick = false) {
        return this.user(user, byNick) + "/guests";
    }
    /**
     * @param user The user id or nick
     * @param byNick Whether the user is by nick (Default: false)
     * @example
     * "/users/:userId/blocked"
     * "/users/by_nick/:userNick/blocked"
     */
    static blocked(user, byNick = false) {
        return this.user(user, byNick) + "/blocked";
    }
    /**
     * @param user The user id or nick
     * @param byNick Whether the user is by nick (Default: false)
     * @example
     * "/users/:userId/subscribers"
     * "/users/by_nick/:userNick/subscribers"
     */
    static subscribers(user, byNick = false) {
        return this.user(user, byNick) + "/subscribers";
    }
    /**
     * @param user The user id or nick
     * @param byNick Whether the user is by nick (Default: false)
     * @example
     * "/users/:userId/subscriptions"
     */
    static subscriptions(user, byNick = false) {
        return this.user(user, byNick) + "/subscriptions";
    }
    /**
     * @example
     * "/content"
     * "/content/:contentId"
     */
    static content(id) {
        return "/content" + (id ? `/${id}` : "");
    }
    /**
     * @param contentId The content id or array of content ids if adding to reads
     * @example
     * "/content/:contentId/smiles"
     */
    static smiles(contentId) {
        return this.content(contentId) + "/smiles";
    }
    /**
     * @param contentId The content id
     * @example
     * "/content/:contentId/republished"
     */
    static republishes(id) {
        return this.content(id) + "/republished";
    }
    /**
     * @example
     * "/content/:contentId/comments"
     */
    static comments(contentId) {
        return this.content(contentId) + "/comments";
    }
    /**
     * @param contentId The content id
     * @param commentId The comment id
     * @example
     * "/content/:contentId/comments/:commentId/replies"
     */
    static replies(contentId, commentId) {
        return this.comments(contentId) + `/${commentId}/replies`;
    }
    /**
     * @param contentId The content id
     * @example
     * "/content/:contentId/comments/attached_content"
     */
    static commentAttachment(contentId) {
        return this.comments(contentId) + "/attached_content";
    }
    /**
     * @param contentId The content id or array of content ids if adding to reads
     * @example
     * "/reads"
     * "/reads/:contentId"
     * "/reads/:contentId1,contentId2..."
     */
    static reads(contentId) {
        if (Array.isArray(contentId)) {
            contentId = contentId.join(",");
        }
        return "/reads" + (contentId ? `/${contentId}` : "");
    }
    /**
     * @param userId The id of the user the ban belongs to
     * @param banId The id of the Ban
     * @example
     * "/users/:userId/bans"
     * "/users/:userId/bans/:banId"
     */
    static bans(userId, banId) {
        return `/users/${userId}/bans` + (banId ? `/${banId}` : "");
    }
}
exports.Endpoints = Endpoints;
Endpoints.account = "/account";
Endpoints.collective = "/feeds/collective";
Endpoints.counters = "/counters";
Endpoints.featured = "/feeds/featured";
Endpoints.feeds = "/feeds";
Endpoints.oauth2 = "/oauth2";
Endpoints.projectElements = "/feeds/project_elements";
Endpoints.subscriptionFeed = "/timelines/home";
Endpoints.timelines = "/timelines";
Endpoints.token = "/oauth2/token";
Endpoints.myComments = "users/my/comments";
Endpoints.chatInvites = "users/my/chat_invitations";
Endpoints.mySmiles = "/users/my/content_smiles";
Endpoints.myAppeals = "users/my/appeals";
Endpoints.unreadChatMessages = "users/my/unread_chat_messages";
Endpoints.myNews = "news/my";
//# sourceMappingURL=Endpoints.js.map