"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const BaseClient_1 = require("./BaseClient");
const ifunny_api_types_1 = require("@ifunny/ifunny-api-types");
const ErrorHandler_1 = require("../errors/ErrorHandler");
const iFunnyError_1 = require("../errors/iFunnyError");
const iFunnyErrorCodes_1 = require("../errors/iFunnyErrorCodes");
const Methods_1 = require("../utils/Methods");
/**
 * Client for the Discord API.
 * @extends BaseClient
 */
class Client extends BaseClient_1.BaseClient {
    constructor(config, payload = {}) {
        super(config, payload);
    }
    get basic() {
        return this._basic;
    }
    set basic(value) {
        this._basic = value;
    }
    /**
     * The bearer token for the client
     */
    get bearer() {
        return this._bearer;
    }
    /**
     * Sets the bearer token for the client. If set to null, the client will not use a bearer token.
     */
    set bearer(value) {
        this._bearer = value || null;
        this.instance.defaults.headers.common.Authorization = this.isAuthorized()
            ? `Bearer ${this.bearer}`
            : `Basic ${this.basic}`;
    }
    /**
     * Is the client using a bearer token to make requests?
     */
    isAuthorized() {
        return !!this.bearer;
    }
    fetch() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isAuthorized()) {
                throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.ClientNotLoggedIn, "Fetch account data");
            }
            try {
                const response = yield this.instance.get(ifunny_api_types_1.Endpoints.account);
                console.log(response);
                let data = response.data;
                this.payload = data.data;
                return this;
            }
            catch (error) {
                if (error instanceof Error && (0, Methods_1.isAxiosError)(error)) {
                    const resp = error.response;
                    //console.log(JSON.stringify(resp, null, 2));
                    switch ((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) {
                        case 401:
                            throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidGrant, resp.data.error_description);
                    }
                    throw error;
                }
                throw error;
            }
        });
    }
    /**
     * Logs into iFunny with an Email and Password\
     * If you already have a bearer token, don't use this method
     * @param email The email to login with
     * @param password The password to login with
     * @returns The instance of the client
     */
    login(email, password) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isAuthorized()) {
                return yield this.fetch();
            }
            try {
                let response = yield this.instance.post(ifunny_api_types_1.Endpoints.token, {
                    grant_type: "password",
                    username: email,
                    password,
                });
                this.bearer = response.data.access_token;
                return yield this.fetch();
            }
            catch (error) {
                if (!(error instanceof Error) || !(0, Methods_1.isAxiosError)(error))
                    throw error;
                const resp = error.response;
                switch (resp.data.error) {
                    case "captcha_required":
                        throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.CaptchaRequired, resp.data);
                    default:
                        throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.UnknownError, error);
                }
            }
        });
    }
    /**
     * Signs up for an iFunny account.
     * @param nick The nick of the account to sign up with. Checks if the nick is taken.
     * @param email The email to sign up with. Checks if the email is taken.
     * @param password The password to sign up with
     * @param login Should the client login after sign up?
     * @returns The instance of the client
     */
    signUp(nick, email, password, acceptMailOffers = false, login = true) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let response = yield this.instance.post(ifunny_api_types_1.Endpoints.user(), {
                    reg_type: "pwd",
                    nick,
                    email,
                    password,
                    accept_mailing: acceptMailOffers ? 1 : 0,
                });
                this.payload.id = response.data.data.id;
                if (login) {
                    return yield this.login(email, password);
                }
                return this;
            }
            catch (error) {
                if (!(error instanceof Error))
                    throw error;
                throw (0, ErrorHandler_1.handleError)(error);
            }
        });
    }
}
exports.Client = Client;
exports.default = Client;
//# sourceMappingURL=Client.js.map