"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Token = exports.BasicRegex = exports.Basic156Regex = exports.Basic112Regex = exports.BearerRegex = exports.capitalize = void 0;
const ifunny_api_types_1 = require("@ifunny/ifunny-api-types");
const Constants_1 = require("../utils/Constants");
const crypto_1 = __importDefault(require("crypto"));
const iFunnyError_1 = require("../errors/iFunnyError");
const iFunnyErrorCodes_1 = require("../errors/iFunnyErrorCodes");
const inspectSymbol = Symbol.for("nodejs.util.inspect.custom");
// Capitalize the first letter of a string.
function capitalize(str) {
    return str.charAt(0).toUpperCase() + str.slice(1);
}
exports.capitalize = capitalize;
exports.BearerRegex = /^[a-z0-9]{64}$/;
exports.Basic112Regex = /^[a-z0-9]{112}$/i;
exports.Basic156Regex = /^[a-z0-9]{156}$/i;
exports.BasicRegex = /^[a-z0-9]{112}|[a-z0-9]{156}$/i;
/**
 * Represents an auth token for the iFunny API.
 */
class Token {
    /**
     * @param token The auth token
     */
    constructor(token, config = {}) {
        var _a, _b, _c;
        this._token = token;
        this._config = {
            type: (_a = config.type) !== null && _a !== void 0 ? _a : this.type,
            id: (_b = config.id) !== null && _b !== void 0 ? _b : Constants_1.DefaultClientId,
            secret: (_c = config.secret) !== null && _c !== void 0 ? _c : Constants_1.DefaultClientSecret,
        };
        if (!this.test(token)) {
            throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidGrant, token);
        }
    }
    get config() {
        return this._config;
    }
    /**
     * The value of the token
     */
    get value() {
        return this._token;
    }
    /**
     * The type of token this is\
     * Inferred from the token length
     */
    get type() {
        switch (this.length) {
            case 64:
                return ifunny_api_types_1.TOKEN_TYPES.BEARER;
            case 112:
            case 156:
                return ifunny_api_types_1.TOKEN_TYPES.BASIC;
            default:
                throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidGrant, this.value);
        }
    }
    /**
     * The length of the token
     */
    get length() {
        return this.value.length;
    }
    /**
     * The regex it should pass to validate the token
     */
    get regex() {
        switch (this.type) {
            case ifunny_api_types_1.TOKEN_TYPES.BEARER:
                return exports.BearerRegex;
            case ifunny_api_types_1.TOKEN_TYPES.BASIC:
                return exports.BasicRegex;
            default:
                throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidTokenType, this.type);
        }
    }
    /**
     * The string representation of the token
     * @example "Bearer <token>"
     * @example "Basic <token>"
     */
    toString() {
        return `${capitalize(this.type)} ${this.value}`;
    }
    [inspectSymbol](depth, opts, inspect) {
        return this.toString();
    }
    /**
     * Validates that the token passes the regex based on it's type `basic` | `bearer`
     * @param token Token to validate against the regex (Default: this.token)
     * @returns Is the token a valid
     */
    test(token) {
        return this.regex.test(token !== null && token !== void 0 ? token : this.value);
    }
    /**
     * Compares a token to itself to see if they are equal
     * @param token Token to compare against
     * @returns
     */
    equals(token) {
        if (typeof token === "string")
            token = new Token(token);
        return this.value === token.value && this.type === token.type;
    }
    /**
     * Generates a basic token
     * @param opts Options to create the token with
     * @param opts.clientId The client id to use for the token
     * @param opts.clientSecret The client secret to use for the token
     * @param opts.length The length of the token to create
     * @returns
     */
    static generateBasic(opts) {
        const { clientId = Constants_1.DefaultClientId, clientSecret = Constants_1.DefaultClientSecret, length = 112, } = (opts !== null && opts !== void 0 ? opts : (opts = {}));
        let uuid = crypto_1.default.randomUUID().replace(/\-/g, "");
        let hex;
        switch (length) {
            case 112:
                hex = uuid.toUpperCase();
                break;
            case 156:
                hex = crypto_1.default
                    .createHash("sha256")
                    .update(uuid)
                    .digest("hex")
                    .toUpperCase();
                break;
            default:
                throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidBasicTokenLength);
        }
        let a = hex + `_${clientId}:`;
        let b = hex + `:${clientId}:${clientSecret}`;
        let c = crypto_1.default.createHash("sha1").update(b).digest("hex");
        let value = Buffer.from(a + c).toString("base64");
        return new Token(value, {
            id: clientId,
            secret: clientSecret,
            type: ifunny_api_types_1.TOKEN_TYPES.BASIC,
        });
    }
}
exports.Token = Token;
console.log(Token.generateBasic());
//# sourceMappingURL=Token.js.map